/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.components;

import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.tools.ui.options.InputUiOptionBuilder;
import fr.exemole.bdfserver.tools.ui.options.UiOptionsUtils;
import fr.exemole.bdfserver.tools.ui.options.ValueUiOptionBuilder;
import net.fichotheque.SubsetKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChangeBuilder;
import net.mapeadores.util.text.Labels;
import fr.exemole.bdfserver.api.ui.options.InputUiOption;
import fr.exemole.bdfserver.api.ui.options.ValueUiOption;


/**
 *
 * @author Vincent Calame
 */
public class IncludeUiBuilder {

    private final String name;
    private final ExtendedIncludeKey extendedIncludeKey;
    private final AttributeChangeBuilder attributeChangeBuilder = new AttributeChangeBuilder();
    private final LabelChangeBuilder labelChangeBuilder = new LabelChangeBuilder();
    private final Attributes previousAttributes;
    private final InputUiOptionBuilder inputUiOptionBuilder;
    private final ValueUiOptionBuilder valueUiOptionBuilder;
    private String status = BdfServerConstants.DEFAULT_STATUS;

    public IncludeUiBuilder(String name) {
        this(name, null);
    }

    public IncludeUiBuilder(String name, @Nullable Attributes previousAttributes) {
        if (name == null) {
            throw new IllegalArgumentException("name is null");
        }
        this.name = name;
        this.extendedIncludeKey = null;
        this.inputUiOptionBuilder = null;
        this.valueUiOptionBuilder = null;
        this.previousAttributes = previousAttributes;
    }

    public IncludeUiBuilder(SpecialIncludeUi specialIncludeUi) {
        this.name = specialIncludeUi.getName();
        this.status = specialIncludeUi.getStatus();
        this.extendedIncludeKey = null;
        this.inputUiOptionBuilder = null;
        this.valueUiOptionBuilder = null;
        this.previousAttributes = specialIncludeUi.getAttributes();
        Labels customLabels = specialIncludeUi.getCustomLabels();
        if (customLabels != null) {
            labelChangeBuilder.putLabels(customLabels);
        }
    }

    public IncludeUiBuilder(ExtendedIncludeKey extendedIncludeKey) {
        this(extendedIncludeKey, null);
    }

    public IncludeUiBuilder(ExtendedIncludeKey extendedIncludeKey, @Nullable Attributes previousAttributes) {
        if (extendedIncludeKey == null) {
            throw new IllegalArgumentException("extendedIncludeKey is null");
        }
        this.name = extendedIncludeKey.getKeyString();
        this.extendedIncludeKey = extendedIncludeKey;
        SubsetKey subsetKey = extendedIncludeKey.getSubsetKey();
        if (SubsetIncludeUi.hasInputUiOption(subsetKey)) {
            this.inputUiOptionBuilder = new InputUiOptionBuilder(UiOptionsUtils.INDEXATION_DEFAULT_INPUTUIOPTION);
        } else {
            this.inputUiOptionBuilder = null;
        }
        if (SubsetIncludeUi.hasValueUiOption(subsetKey)) {
            this.valueUiOptionBuilder = new ValueUiOptionBuilder();
        } else {
            this.valueUiOptionBuilder = null;
        }
        this.previousAttributes = previousAttributes;
    }

    public IncludeUiBuilder(SubsetIncludeUi subsetIncludeUi) {
        this.name = subsetIncludeUi.getName();
        this.extendedIncludeKey = subsetIncludeUi.getExtendedIncludeKey();
        this.status = subsetIncludeUi.getStatus();
        InputUiOption inputUiOption = subsetIncludeUi.getInputUiOption();
        if (inputUiOption != null) {
            this.inputUiOptionBuilder = new InputUiOptionBuilder(inputUiOption);
        } else {
            this.inputUiOptionBuilder = null;
        }
        ValueUiOption valueUiOption = subsetIncludeUi.getValueUiOption();
        if (valueUiOption != null) {
            this.valueUiOptionBuilder = new ValueUiOptionBuilder(valueUiOption);
        } else {
            this.valueUiOptionBuilder = null;
        }
        this.previousAttributes = subsetIncludeUi.getAttributes();
        Labels customLabels = subsetIncludeUi.getCustomLabels();
        if (customLabels != null) {
            for (Label label : customLabels) {
                labelChangeBuilder.putLabel(label);
            }
        }
    }

    public IncludeUiBuilder setStatus(String status) {
        if (status != null) {
            this.status = BdfServerConstants.checkValidStatus(status);
        }
        return this;
    }

    public InputUiOptionBuilder getInputUiOptionBuilder() {
        return inputUiOptionBuilder;
    }

    public ValueUiOptionBuilder getValueUiOptionBuilder() {
        return valueUiOptionBuilder;
    }

    public AttributeChangeBuilder getAttributeChangeBuilder() {
        return attributeChangeBuilder;
    }

    public LabelChangeBuilder getLabelChangeBuilder() {
        return labelChangeBuilder;
    }

    public IncludeUi toIncludeUi() {
        Labels customLabels = labelChangeBuilder.toLabels();
        if (customLabels.isEmpty()) {
            customLabels = null;
        }
        if (extendedIncludeKey == null) {
            return new InternalSpecialIncludeUi(name, status, customLabels, attributeChangeBuilder.toAttributes(previousAttributes));
        } else {
            InputUiOption inputUiOption = null;
            ValueUiOption valueUiOption = null;
            if (inputUiOptionBuilder != null) {
                inputUiOption = inputUiOptionBuilder.toInputUiOption();
            }
            if (valueUiOptionBuilder != null) {
                valueUiOption = valueUiOptionBuilder.toValueUiOption();
            }
            return new InternalSubsetIncludeUi(name, status, extendedIncludeKey, inputUiOption, valueUiOption, customLabels, attributeChangeBuilder.toAttributes(previousAttributes));
        }
    }

    public static IncludeUiBuilder initSubset(ExtendedIncludeKey includeKey) {
        return new IncludeUiBuilder(includeKey, null);
    }

    public static IncludeUiBuilder initSubset(ExtendedIncludeKey includeKey, @Nullable Attributes attributes) {
        return new IncludeUiBuilder(includeKey, attributes);
    }

    public static IncludeUiBuilder initSpecial(String specialIncludeName) {
        return new IncludeUiBuilder(specialIncludeName, null);
    }

    public static IncludeUiBuilder initSpecial(String specialIncludeName, @Nullable Attributes attributes) {
        return new IncludeUiBuilder(specialIncludeName, attributes);
    }

    public static IncludeUiBuilder init(IncludeUi includeUi) {
        if (includeUi instanceof SubsetIncludeUi) {
            return new IncludeUiBuilder((SubsetIncludeUi) includeUi);
        }
        if (includeUi instanceof SpecialIncludeUi) {
            return new IncludeUiBuilder((SpecialIncludeUi) includeUi);
        }
        throw new ClassCastException("unknown implementation: " + includeUi.getClass().getCanonicalName());
    }


    private static class InternalSpecialIncludeUi implements SpecialIncludeUi {

        private final String name;
        private final String status;
        private final Labels customLabels;
        private final Attributes attributes;

        private InternalSpecialIncludeUi(String name, String status, Labels customLabels, Attributes attributes) {
            this.name = name;
            this.status = status;
            this.customLabels = customLabels;
            this.attributes = attributes;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getStatus() {
            return status;
        }

        @Override
        public Labels getCustomLabels() {
            return customLabels;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalSubsetIncludeUi implements SubsetIncludeUi {

        private final String name;
        private final ExtendedIncludeKey extendedIncludeKey;
        private final String status;
        private final InputUiOption inputUiOption;
        private final ValueUiOption valueUiOption;
        private final Labels customLabels;
        private final Attributes attributes;

        private InternalSubsetIncludeUi(String name, String status, ExtendedIncludeKey extendedIncludeKey, InputUiOption inputUiOption, ValueUiOption valueUiOption, Labels customLabels, Attributes attributes) {
            this.name = name;
            this.status = status;
            this.extendedIncludeKey = extendedIncludeKey;
            this.inputUiOption = inputUiOption;
            this.valueUiOption = valueUiOption;
            this.customLabels = customLabels;
            this.attributes = attributes;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getStatus() {
            return status;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return extendedIncludeKey;
        }

        @Override
        public InputUiOption getInputUiOption() {
            return inputUiOption;
        }

        @Override
        public ValueUiOption getValueUiOption() {
            return valueUiOption;
        }

        @Override
        public Labels getCustomLabels() {
            return customLabels;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }

}
