/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.options;

import fr.exemole.bdfserver.api.ui.options.InputUiOption;
import fr.exemole.bdfserver.api.ui.options.ValueUiOption;
import java.util.function.Consumer;
import net.fichotheque.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class UiOptionsUtils {

    public final static ValueUiOption DEFAULT_VALUEUIOPTION = ValueUiOptionBuilder.init().toValueUiOption();
    public final static InputUiOption PROPRIETE_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_MEDIUM, 1);
    public final static InputUiOption SECTION_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_LARGE, 8);
    public final static InputUiOption INFORMATION_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_LARGE, 1);
    public final static InputUiOption TITRE_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_LARGE, 1);
    public final static InputUiOption REDACTEURS_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_SMALL, 1);
    public final static InputUiOption INDEXATION_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_TEXT, InputUiOption.WIDTH_LARGE, 1);
    public final static InputUiOption LANG_DEFAULT_INPUTUIOPTION = InputUiOptionBuilder.build(InputUiOption.INPUT_SELECT, InputUiOption.WIDTH_SMALL, 1);

    private UiOptionsUtils() {

    }

    public static InputUiOption getDefaultInputUiOption(FieldKey fieldKey) {
        switch (fieldKey.getCategory()) {
            case FieldKey.PROPRIETE_CATEGORY:
                return PROPRIETE_DEFAULT_INPUTUIOPTION;
            case FieldKey.INFORMATION_CATEGORY:
                return INFORMATION_DEFAULT_INPUTUIOPTION;
            case FieldKey.SECTION_CATEGORY:
                return SECTION_DEFAULT_INPUTUIOPTION;
            case FieldKey.SPECIAL_CATEGORY:
                switch (fieldKey.getKeyString()) {
                    case FieldKey.SPECIAL_TITRE:
                    case FieldKey.SPECIAL_SOUSTITRE:
                        return TITRE_DEFAULT_INPUTUIOPTION;
                    case FieldKey.SPECIAL_REDACTEURS:
                        return REDACTEURS_DEFAULT_INPUTUIOPTION;
                    case FieldKey.SPECIAL_LANG:
                        return LANG_DEFAULT_INPUTUIOPTION;
                    case FieldKey.SPECIAL_ID:
                        throw new IllegalArgumentException("No InputUiOption for FieldKey.ID");
                    default:
                        throw new SwitchException("Unknown special key = " + fieldKey.getKeyString());
                }
            default:
                throw new SwitchException("unknown category = " + fieldKey.getCategory());
        }
    }

    public static void readValueUiOption(Element element, ValueUiOptionBuilder valueUiOptionBuilder, MessageHandler messageHandler) {
        if (valueUiOptionBuilder == null) {
            return;
        }
        String defaultValue = element.getAttribute("default");
        if (defaultValue.length() != 0) {
            valueUiOptionBuilder.setDefaultValue(defaultValue);
        }
        DOMUtils.readChildren(element, new ValueConsumer(valueUiOptionBuilder, messageHandler));
    }

    public static void readInputUiOption(Element element, InputUiOptionBuilder inputUiOptionBuilder, MessageHandler messageHandler) {
        if (inputUiOptionBuilder == null) {
            return;
        }
        String xpath = "input";
        String inputType = element.getAttribute("type");
        if (inputType.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "type");
        } else {
            try {
                inputUiOptionBuilder.setInputType(inputType);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "type", inputType);
            }
        }
        String widthType = element.getAttribute("width");
        if (widthType.length() == 0) {
            widthType = element.getAttribute("size"); //Ancien nom de l'attribut
        }
        if (widthType.length() > 0) {
            try {
                inputUiOptionBuilder.setWidthType(widthType);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "width", widthType);
            }
        }
        String rowsString = element.getAttribute("rows");
        if (rowsString.length() > 0) {
            try {
                int row = Integer.parseInt(rowsString);
                inputUiOptionBuilder.setRows(row);
            } catch (NumberFormatException nfe) {
                DomMessages.wrongIntegerAttributeValue(messageHandler, xpath, "rows", rowsString);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "rows", rowsString);
            }
        }
    }


    private static class ValueConsumer implements Consumer<Element> {

        private final ValueUiOptionBuilder valueUiOptionBuilder;
        private final MessageHandler messageHandler;


        private ValueConsumer(ValueUiOptionBuilder valueUiOptionBuilder, MessageHandler messageHandler) {
            this.valueUiOptionBuilder = valueUiOptionBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("default")) {
                valueUiOptionBuilder.setDefaultValue(XMLUtils.getRawData(element));
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }


}
